@@ -1,11 +1,11 @@
-from typing import Any, Callable, Optional, Type, TypedDict, TypeVar, Union
+from typing import Any, Callable, Optional, TypedDict, TypeVar, Union
 
 from pydantic_compat import PYDANTIC2, Field, model_validator
 
 from app_model import expressions
 
 from ._base import _BaseModel
-from ._constants import OperatingSystem
+from ._constants import KeyBindingSource, OperatingSystem
 from ._keys import StandardKeyBinding
 
 KeyEncoding = Union[int, str]
@@ -29,26 +29,30 @@ class KeyBindingRule(_BaseModel):
     """
 
     primary: Optional[KeyEncoding] = Field(
-        None, description="(Optional) Key combo, (e.g. Ctrl+O)."
+        default=None, description="(Optional) Key combo, (e.g. Ctrl+O)."
     )
     win: Optional[KeyEncoding] = Field(
-        None, description="(Optional) Windows specific key combo."
+        default=None, description="(Optional) Windows specific key combo."
     )
     mac: Optional[KeyEncoding] = Field(
-        None, description="(Optional) MacOS specific key combo."
+        default=None, description="(Optional) MacOS specific key combo."
     )
     linux: Optional[KeyEncoding] = Field(
-        None, description="(Optional) Linux specific key combo."
+        default=None, description="(Optional) Linux specific key combo."
     )
     when: Optional[expressions.Expr] = Field(
-        None,
+        default=None,
         description="(Optional) Condition when the keybingding is active.",
     )
     weight: int = Field(
-        0,
+        default=0,
         description="Internal weight used to sort keybindings. "
         "This is not part of the plugin schema",
     )
+    source: KeyBindingSource = Field(
+        default=KeyBindingSource.APP,
+        description="Who registered the keybinding. Used to sort keybindings.",
+    )
 
     def _bind_to_current_platform(self) -> Optional[KeyEncoding]:
         if _WIN and self.win:
@@ -66,10 +70,10 @@ class KeyBindingRule(_BaseModel):
     # so sticking with this one conditional method here...
     if PYDANTIC2:
         # for v2
-        @model_validator(mode="wrap")  # type: ignore
+        @model_validator(mode="wrap")
         @classmethod
         def _model_val(
-            cls: Type[M], v: Any, handler: Callable[[Any], M]
+            cls: type[M], v: Any, handler: Callable[[Any], M]
         ) -> "KeyBindingRule":
             if isinstance(v, StandardKeyBinding):
                 return v.to_keybinding_rule()
